@@ -55,7 +55,7 @@
 #include "str.h"
 #include "others.h"
 
-// shared
+// common
 #include "optparser.h"
 #include "output.h"
 #include "misc.h"
@@ -88,7 +88,6 @@ static struct {
     /* must be before VERSION, because they share common prefix! */
     {CMD18, sizeof(CMD18) - 1, COMMAND_COMMANDS, 0, 0, 1},
     {CMD7, sizeof(CMD7) - 1, COMMAND_VERSION, 0, 1, 1},
-    {CMD8, sizeof(CMD8) - 1, COMMAND_STREAM, 0, 1, 1},
     {CMD10, sizeof(CMD10) - 1, COMMAND_END, 0, 0, 1},
     {CMD11, sizeof(CMD11) - 1, COMMAND_SHUTDOWN, 0, 1, 1},
     {CMD13, sizeof(CMD13) - 1, COMMAND_MULTISCAN, 1, 1, 1},
@@ -110,19 +109,19 @@ enum commands parse_command(const char *cmd, const char **argument, int oldstyle
             const char *arg = cmd + len;
             if (commands[i].need_arg) {
                 if (!*arg) { /* missing argument */
-                    logg("$Command %s missing argument!\n", commands[i].cmd);
+                    logg(LOGG_DEBUG_NV, "Command %s missing argument!\n", commands[i].cmd);
                     return COMMAND_UNKNOWN;
                 }
                 *argument = arg + 1;
             } else {
                 if (*arg) { /* extra stuff after command */
-                    logg("$Command %s has trailing garbage!\n", commands[i].cmd);
+                    logg(LOGG_DEBUG_NV, "Command %s has trailing garbage!\n", commands[i].cmd);
                     return COMMAND_UNKNOWN;
                 }
                 *argument = NULL;
             }
             if (oldstyle && !commands[i].support_old) {
-                logg("$Command sent as old-style when not supported: %s\n", commands[i].cmd);
+                logg(LOGG_DEBUG_NV, "Command sent as old-style when not supported: %s\n", commands[i].cmd);
                 return COMMAND_UNKNOWN;
             }
             return commands[i].cmdtype;
@@ -190,15 +189,17 @@ int conn_reply_errno(const client_conn_t *conn, const char *path,
  */
 int command(client_conn_t *conn, int *virus)
 {
-    int desc                        = conn->sd;
-    struct cl_engine *engine        = conn->engine;
-    struct cl_scan_options *options = conn->options;
-    const struct optstruct *opts    = conn->opts;
-    enum scan_type type             = TYPE_INIT;
+    int desc                 = conn->sd;
+    struct cl_engine *engine = conn->engine;
+    struct cl_scan_options options;
+    const struct optstruct *opts = conn->opts;
+    enum scan_type type          = TYPE_INIT;
     int maxdirrec;
     int ret   = 0;
     int flags = CLI_FTW_STD;
 
+    memcpy(&options, conn->options, sizeof(struct cl_scan_options));
+
     struct scan_cb_data scandata;
     struct cli_ftw_cbdata data;
     unsigned ok, error, total;
@@ -206,7 +207,7 @@ int command(client_conn_t *conn, int *virus)
     jobgroup_t *group = NULL;
 
     if (thrmgr_group_need_terminate(conn->group)) {
-        logg("$Client disconnected while command was active\n");
+        logg(LOGG_DEBUG_NV, "Client disconnected while command was active\n");
         if (conn->scanfd != -1)
             close(conn->scanfd);
         return 1;
@@ -219,7 +220,7 @@ int command(client_conn_t *conn, int *virus)
     scandata.group         = conn->group;
     scandata.odesc         = desc;
     scandata.conn          = conn;
-    scandata.options       = options;
+    scandata.options       = &options;
     scandata.engine        = engine;
     scandata.opts          = opts;
     scandata.thr_pool      = conn->thrpool;
@@ -257,9 +258,9 @@ int command(client_conn_t *conn, int *virus)
             pthread_mutex_unlock(&conn->thrpool->pool_mutex);
             if (ret) {
                 /* multiscan has 1 control thread, so there needs to be at least
-		   1 threads that is a non-multiscan controlthread to scan and
-		   make progress. */
-                logg("^Not enough threads for multiscan. Max: %d, Alive: %d, Multiscan: %d+1\n",
+                   1 threads that is a non-multiscan controlthread to scan and
+                   make progress. */
+                logg(LOGG_WARNING, "Not enough threads for multiscan. Max: %d, Alive: %d, Multiscan: %d+1\n",
                      max, alive, multiscan);
                 conn_reply(conn, conn->filename, "Not enough threads for multiscan. Increase MaxThreads.", "ERROR");
                 return 1;
@@ -297,7 +298,7 @@ int command(client_conn_t *conn, int *virus)
                 conn_reply_error(conn, "FILDES: didn't receive file descriptor.");
                 return 1;
             } else {
-                ret = scanfd(conn, NULL, engine, options, opts, desc, 0);
+                ret = scanfd(conn, NULL, engine, &options, opts, desc, 0);
                 if (ret == CL_VIRUS) {
                     *virus = 1;
                     ret    = 0;
@@ -311,7 +312,7 @@ int command(client_conn_t *conn, int *virus)
                     ret = 1;
                 } else
                     ret = 0;
-                logg("$Closed fd %d\n", conn->scanfd);
+                logg(LOGG_DEBUG_NV, "Closed fd %d\n", conn->scanfd);
                 close(conn->scanfd);
             }
             return ret;
@@ -326,21 +327,9 @@ int command(client_conn_t *conn, int *virus)
                 mdprintf(desc, "%u: ", conn->id);
             thrmgr_printstats(desc, conn->term);
             return 0;
-        case COMMAND_STREAM:
-            thrmgr_setactivetask(NULL, "STREAM");
-            ret = scanstream(desc, NULL, engine, options, opts, conn->term);
-            if (ret == CL_VIRUS)
-                *virus = 1;
-            if (ret == CL_EMEM) {
-                if (optget(opts, "ExitOnOOM")->enabled)
-                    return -1;
-                else
-                    return 1;
-            }
-            return 0;
         case COMMAND_INSTREAMSCAN:
             thrmgr_setactivetask(NULL, "INSTREAM");
-            ret = scanfd(conn, NULL, engine, options, opts, desc, 1);
+            ret = scanfd(conn, NULL, engine, &options, opts, desc, 1);
             if (ret == CL_VIRUS) {
                 *virus = 1;
                 ret    = 0;
@@ -356,7 +345,7 @@ int command(client_conn_t *conn, int *virus)
                 ret = 0;
             if (ftruncate(conn->scanfd, 0) == -1) {
                 /* not serious, we're going to close it and unlink it anyway */
-                logg("*ftruncate failed: %d\n", errno);
+                logg(LOGG_DEBUG, "ftruncate failed: %d\n", errno);
             }
             close(conn->scanfd);
             conn->scanfd = -1;
@@ -364,7 +353,7 @@ int command(client_conn_t *conn, int *virus)
             return ret;
         case COMMAND_ALLMATCHSCAN:
             if (!optget(opts, "AllowAllMatchScan")->enabled) {
-                logg("$Rejecting ALLMATCHSCAN command.\n");
+                logg(LOGG_DEBUG_NV, "Rejecting ALLMATCHSCAN command.\n");
                 conn_reply(conn, conn->filename, "ALLMATCHSCAN command disabled by clamd configuration.", "ERROR");
                 return 1;
             }
@@ -373,7 +362,7 @@ int command(client_conn_t *conn, int *virus)
             type = TYPE_SCAN;
             break;
         default:
-            logg("!Invalid command dispatched: %d\n", conn->cmdtype);
+            logg(LOGG_ERROR, "Invalid command dispatched: %d\n", conn->cmdtype);
             return 1;
     }
 
@@ -424,13 +413,13 @@ static int dispatch_command(client_conn_t *conn, enum commands cmd, const char *
     client_conn_t *dup_conn = (client_conn_t *)malloc(sizeof(struct client_conn_tag));
 
     if (!dup_conn) {
-        logg("!Can't allocate memory for client_conn\n");
+        logg(LOGG_ERROR, "Can't allocate memory for client_conn\n");
         return -1;
     }
     memcpy(dup_conn, conn, sizeof(*conn));
     dup_conn->cmdtype = cmd;
     if (cl_engine_addref(dup_conn->engine)) {
-        logg("!cl_engine_addref() failed\n");
+        logg(LOGG_ERROR, "cl_engine_addref() failed\n");
         free(dup_conn);
         return -1;
     }
@@ -452,7 +441,7 @@ static int dispatch_command(client_conn_t *conn, enum commands cmd, const char *
         case COMMAND_ALLMATCHSCAN:
             dup_conn->filename = cli_strdup_to_utf8(argument);
             if (!dup_conn->filename) {
-                logg("!Failed to allocate memory for filename\n");
+                logg(LOGG_ERROR, "Failed to allocate memory for filename\n");
                 ret = -1;
             }
             break;
@@ -460,21 +449,20 @@ static int dispatch_command(client_conn_t *conn, enum commands cmd, const char *
             dup_conn->scanfd = conn->scanfd;
             conn->scanfd     = -1;
             break;
-        case COMMAND_STREAM:
         case COMMAND_STATS:
             /* not a scan command, don't queue to bulk */
             bulk = 0;
             /* just dispatch the command */
             break;
         default:
-            logg("!Invalid command dispatch: %d\n", cmd);
+            logg(LOGG_ERROR, "Invalid command dispatch: %d\n", cmd);
             ret = -2;
             break;
     }
     if (!dup_conn->group)
         bulk = 0;
     if (!ret && !thrmgr_group_dispatch(dup_conn->thrpool, dup_conn->group, dup_conn, bulk)) {
-        logg("!thread dispatch failed\n");
+        logg(LOGG_ERROR, "thread dispatch failed\n");
         ret = -2;
     }
     if (ret) {
@@ -555,7 +543,7 @@ int execute_or_dispatch_command(client_conn_t *conn, enum commands cmd, const ch
             default:
                 /* these commands are not recognized inside an IDSESSION */
                 conn_reply_error(conn, "Command invalid inside IDSESSION.");
-                logg("$SESSION: command is not valid inside IDSESSION: %d\n", cmd);
+                logg(LOGG_DEBUG_NV, "SESSION: command is not valid inside IDSESSION: %d\n", cmd);
                 conn->group = NULL;
                 return 1;
         }
@@ -573,7 +561,7 @@ int execute_or_dispatch_command(client_conn_t *conn, enum commands cmd, const ch
             pthread_mutex_unlock(&reload_mutex);
             mdprintf(desc, "RELOADING%c", term);
             /* we set reload flag, and we'll reload before closing the
-	     * connection */
+             * connection */
             return 1;
         case COMMAND_PING:
             if (conn->group)
@@ -609,7 +597,6 @@ int execute_or_dispatch_command(client_conn_t *conn, enum commands cmd, const ch
             conn->mode  = MODE_STREAM;
             return 0;
         }
-        case COMMAND_STREAM:
         case COMMAND_MULTISCAN:
         case COMMAND_CONTSCAN:
         case COMMAND_STATS:
