@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB, Mickey Sola
@@ -57,11 +57,12 @@
 #include "str.h"
 #include "others.h"
 
-// shared
+// common
 #include "optparser.h"
 #include "output.h"
 #include "misc.h"
 #include "actions.h"
+#include "clamdcom.h"
 
 #include "communication.h"
 #include "client.h"
@@ -70,8 +71,6 @@
 
 #include "../clamonacc.h"
 
-struct sockaddr_un nixsock;
-
 void onas_print_server_version(struct onas_context **ctx)
 {
     if (onas_get_clamd_version(ctx)) {
@@ -90,6 +89,7 @@ int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
     CURLcode curlcode;
     char *ipaddr = NULL;
     int64_t timeout;
+    const char zPING[] = "zPING";
 
     timeout = optget((*ctx)->clamdopts, "OnAccessCurlTimeout")->numarg;
 
@@ -118,7 +118,7 @@ int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
         }
 
         if (NULL == ipaddr) {
-            logg("!ClamClient: Clamonacc does not support binding to INADDR_ANY, \
+            logg(LOGG_ERROR, "ClamClient: Clamonacc does not support binding to INADDR_ANY, \
 					please specify an address with TCPAddr in your clamd.conf config file\n");
             *err = CL_EARG;
             return ret;
@@ -126,7 +126,7 @@ int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
 
         curlcode = onas_curl_init(&curl, ipaddr, (*ctx)->portnum, timeout);
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not init curl, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not init curl, %s\n", curl_easy_strerror(curlcode));
             *err = CL_EARG;
             return ret;
         }
@@ -134,17 +134,18 @@ int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
         curlcode = curl_easy_perform(curl);
         if (CURLE_OK != curlcode) {
             if (optget((*ctx)->opts, "ping")->enabled || optget((*ctx)->opts, "wait")->enabled) {
-                logg("~ClamClient: Initial connection failed, %s. Will try again...\n", curl_easy_strerror(curlcode));
+                logg(LOGG_INFO, "ClamClient: Initial connection failed, %s. Will try again...\n", curl_easy_strerror(curlcode));
             } else {
-                logg("!ClamClient: Could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
+                logg(LOGG_ERROR, "ClamClient: Could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
             }
+            curl_easy_cleanup(curl);
             *err = CL_EARG;
             return ret;
         }
 
 #ifndef ONAS_DEBUG
-        if (onas_sendln(curl, "zPING", 5, timeout)) {
-            logg("!ClamClient: could not ping clamd, %s\n", curl_easy_strerror(curlcode));
+        if (onas_sendln(curl, zPING, sizeof(zPING), timeout)) {
+            logg(LOGG_ERROR, "ClamClient: could not ping clamd, %s\n", curl_easy_strerror(curlcode));
             *err = CL_EARG;
             curl_easy_cleanup(curl);
             return ret;
@@ -177,9 +178,10 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
     int b_remote   = 0;
     uint16_t ret   = 0;
     int64_t timeout;
+    const char zPING[] = "zPING";
 
     if (ctx == NULL) {
-        logg("!null parameter was passed\n");
+        logg(LOGG_ERROR, "null parameter was passed\n");
         return -1;
     }
 
@@ -187,7 +189,7 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
 
     b_remote = onas_check_remote(ctx, &err);
     if (CL_SUCCESS != err) {
-        logg("*ClamClient: could not check to see if daemon was remote... PINGing again...\n");
+        logg(LOGG_DEBUG, "ClamClient: could not check to see if daemon was remote... PINGing again...\n");
     }
 
     if (!b_remote) {
@@ -195,7 +197,7 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
     } else {
         curlcode = onas_curl_init(&curl, optget((*ctx)->clamdopts, "TCPAddr")->strarg, (*ctx)->portnum, timeout);
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not setup curl with tcp address and port, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not setup curl with tcp address and port, %s\n", curl_easy_strerror(curlcode));
             /* curl cleanup done in onas_curl_init on error */
             ret = -1;
             goto done;
@@ -209,7 +211,7 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
         attempt_str = cli_strdup(opt->strarg);
         if (attempt_str) {
             if (NULL == attempt_str) {
-                logg("!could not allocate memory for string\n");
+                logg(LOGG_ERROR, "could not allocate memory for string\n");
                 ret = -1;
                 goto done;
             }
@@ -219,7 +221,7 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
                 interval_str++;
                 interval = cli_strntoul(interval_str, strlen(interval_str), &errchk, 10);
                 if (interval_str + strlen(interval_str) > errchk) {
-                    logg("^interval_str would go past end of buffer\n");
+                    logg(LOGG_WARNING, "interval_str would go past end of buffer\n");
                     ret = -1;
                     goto done;
                 }
@@ -228,7 +230,7 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
             }
             attempts = cli_strntoul(attempt_str, strlen(attempt_str), &errchk, 10);
             if (attempt_str + strlen(attempt_str) > errchk) {
-                logg("^attempt_str would go past end of buffer\n");
+                logg(LOGG_WARNING, "attempt_str would go past end of buffer\n");
                 ret = -1;
                 goto done;
             }
@@ -241,13 +243,13 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
     do {
         curlcode = curl_easy_perform(curl);
         if (CURLE_OK != curlcode) {
-            logg("*ClamClient: could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
-        } else if (CURLE_OK == onas_sendln(curl, "zPING", 5, timeout)) {
+            logg(LOGG_DEBUG, "ClamClient: could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
+        } else if (CURLE_OK == onas_sendln(curl, zPING, sizeof(zPING), timeout)) {
 
             if (!optget((*ctx)->opts, "wait")->enabled) {
-                logg("PONG\n");
+                logg(LOGG_INFO, "PONG\n");
             } else {
-                logg("*ClamClient: Connected.\n");
+                logg(LOGG_DEBUG, "ClamClient: Connected.\n");
             }
 
             ret = 0;
@@ -257,14 +259,14 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
         if (i + 1 < attempts) {
             if (optget((*ctx)->opts, "wait")->enabled) {
                 if (interval == 1)
-                    logg("*Will try again in %lu second\n", interval);
+                    logg(LOGG_DEBUG, "Will try again in %lu second\n", interval);
                 else
-                    logg("*Will try again in %lu seconds\n", interval);
+                    logg(LOGG_DEBUG, "Will try again in %lu seconds\n", interval);
             } else {
                 if (interval == 1)
-                    logg("PINGing again in %lu second\n", interval);
+                    logg(LOGG_INFO, "PINGing again in %lu second\n", interval);
                 else
-                    logg("PINGing again in %lu seconds\n", interval);
+                    logg(LOGG_INFO, "PINGing again in %lu seconds\n", interval);
             }
             sleep(interval);
         }
@@ -274,9 +276,9 @@ int16_t onas_ping_clamd(struct onas_context **ctx)
     /* timed out */
     ret = 1;
     if (optget((*ctx)->opts, "wait")->enabled) {
-        logg("Wait timeout exceeded; Could not connect to clamd\n");
+        logg(LOGG_INFO, "Wait timeout exceeded; Could not connect to clamd\n");
     } else {
-        logg("PING timeout exceeded; No response from clamd\n");
+        logg(LOGG_INFO, "PING timeout exceeded; No response from clamd\n");
     }
 
 done:
@@ -309,7 +311,7 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
     CURLcode curlcode = CURLE_OK;
 
     if (!curl || !ipaddr) {
-        logg("!ClamClient: invalid (NULL) args passed to onas_curl_init\n");
+        logg(LOGG_ERROR, "ClamClient: invalid (NULL) args passed to onas_curl_init\n");
         return CURLE_FAILED_INIT;
     }
 
@@ -323,14 +325,14 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
         curlcode = curl_easy_setopt(*curl, CURLOPT_UNIX_SOCKET_PATH, ipaddr);
 #endif
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not setup curl with local unix socket, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not setup curl with local unix socket, %s\n", curl_easy_strerror(curlcode));
             curl_easy_cleanup(*curl);
             return curlcode;
         }
 
         curlcode = curl_easy_setopt(*curl, CURLOPT_URL, "http://localhost/");
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not setup curl with local address, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not setup curl with local address, %s\n", curl_easy_strerror(curlcode));
             curl_easy_cleanup(*curl);
             return curlcode;
         }
@@ -339,14 +341,14 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
 
         curlcode = curl_easy_setopt(*curl, CURLOPT_PORT, port);
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not setup curl with tcp port, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not setup curl with tcp port, %s\n", curl_easy_strerror(curlcode));
             curl_easy_cleanup(*curl);
             return curlcode;
         }
 
         curlcode = curl_easy_setopt(*curl, CURLOPT_URL, ipaddr);
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not setup curl with tcp address, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not setup curl with tcp address, %s\n", curl_easy_strerror(curlcode));
             curl_easy_cleanup(*curl);
             return curlcode;
         }
@@ -354,14 +356,14 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
 
     curlcode = curl_easy_setopt(*curl, CURLOPT_NOSIGNAL, 1L);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not setup curl to not use signals, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not setup curl to not use signals, %s\n", curl_easy_strerror(curlcode));
         curl_easy_cleanup(*curl);
         return curlcode;
     }
 
     curlcode = curl_easy_setopt(*curl, CURLOPT_CONNECTTIMEOUT_MS, (long)timeout);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not setup curl with connect timeout, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not setup curl with connect timeout, %s\n", curl_easy_strerror(curlcode));
         curl_easy_cleanup(*curl);
         return curlcode;
     }
@@ -369,7 +371,7 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
     /* we implement our own transfer protocol via send and recv, so we only need to connect */
     curlcode = curl_easy_setopt(*curl, CURLOPT_CONNECT_ONLY, 1L);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not setup curl to connect only, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not setup curl to connect only, %s\n", curl_easy_strerror(curlcode));
         curl_easy_cleanup(*curl);
         return curlcode;
     }
@@ -377,7 +379,7 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
 #ifdef ONAS_DEBUG
     curlcode = curl_easy_setopt(*curl, CURLOPT_VERBOSE, 1L);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not tell curl to be verbose, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not tell curl to be verbose, %s\n", curl_easy_strerror(curlcode));
         curl_easy_cleanup(*curl);
         return curlcode;
     }
@@ -386,18 +388,17 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
     /* don't care about the body of the return message */
     curlcode = curl_easy_setopt(*curl, CURLOPT_NOBODY, 1L);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not setup curl to send HEAD request, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not setup curl to send HEAD request, %s\n", curl_easy_strerror(curlcode));
         curl_easy_cleanup(*curl);
         return curlcode;
     }
 
     curlcode = curl_easy_setopt(*curl, CURLOPT_HEADER, 0L);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not setup curl to not send header, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not setup curl to not send header, %s\n", curl_easy_strerror(curlcode));
         curl_easy_cleanup(*curl);
         return curlcode;
     }
-
     return curlcode;
 }
 
@@ -417,18 +418,6 @@ cl_error_t onas_setup_client(struct onas_context **ctx)
         (*ctx)->printinfected = 1;
     }
 
-    /* initialize logger */
-
-    if ((opt = optget(opts, "log"))->enabled) {
-        logg_file = opt->strarg;
-        if (logg("--------------------------------------\n")) {
-            logg("!ClamClient: problem with internal logger\n");
-            return CL_EARG;
-        }
-    } else {
-        logg_file = NULL;
-    }
-
     if (actsetup(opts)) {
         return CL_EARG;
     }
@@ -449,23 +438,23 @@ cl_error_t onas_setup_client(struct onas_context **ctx)
         if (onas_set_sock_only_once(*ctx) == CL_EWRITE) {
             return CL_EWRITE;
         }
-        logg("*ClamClient: client setup to scan via fd passing\n");
+        logg(LOGG_DEBUG, "ClamClient: client setup to scan via fd passing\n");
         (*ctx)->scantype = FILDES;
         (*ctx)->session  = optget(opts, "multiscan")->enabled;
     } else
 #endif
         if (remote) {
-        logg("*ClamClient: client setup to scan via streaming\n");
+        logg(LOGG_DEBUG, "ClamClient: client setup to scan via streaming\n");
         (*ctx)->scantype = STREAM;
         (*ctx)->session  = optget(opts, "multiscan")->enabled;
     } else if (optget(opts, "multiscan")->enabled) {
-        logg("*ClamClient: client setup to scan in multiscan mode\n");
+        logg(LOGG_DEBUG, "ClamClient: client setup to scan in multiscan mode\n");
         (*ctx)->scantype = MULTI;
     } else if (optget(opts, "allmatch")->enabled) {
-        logg("*ClamClient: client setup to scan in all-match mode\n");
+        logg(LOGG_DEBUG, "ClamClient: client setup to scan in all-match mode\n");
         (*ctx)->scantype = ALLMATCH;
     } else {
-        logg("*ClamClient: client setup for continuous scanning\n");
+        logg(LOGG_DEBUG, "ClamClient: client setup for continuous scanning\n");
         (*ctx)->scantype = CONT;
     }
 
@@ -484,12 +473,13 @@ int onas_get_clamd_version(struct onas_context **ctx)
     int len;
     struct onas_rcvln rcv;
     int64_t timeout;
+    const char zVERSION[] = "zVERSION";
 
     timeout = optget((*ctx)->clamdopts, "OnAccessCurlTimeout")->numarg;
 
     b_remote = onas_check_remote(ctx, &err);
     if (CL_SUCCESS != err) {
-        logg("!ClamClient: could not check to see if daemon was remote\n");
+        logg(LOGG_ERROR, "ClamClient: could not check to see if daemon was remote\n");
         return 2;
     }
 
@@ -498,7 +488,7 @@ int onas_get_clamd_version(struct onas_context **ctx)
     } else {
         curlcode = onas_curl_init(&curl, optget((*ctx)->clamdopts, "TCPAddr")->strarg, (*ctx)->portnum, timeout);
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not setup curl with tcp address and port, %s\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: could not setup curl with tcp address and port, %s\n", curl_easy_strerror(curlcode));
             /* curl cleanup done in onas_curl_init on error */
             return 2;
         }
@@ -508,18 +498,19 @@ int onas_get_clamd_version(struct onas_context **ctx)
 
     curlcode = curl_easy_perform(curl);
     if (CURLE_OK != curlcode) {
-        logg("*ClamClient: could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_DEBUG, "ClamClient: could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
+        curl_easy_cleanup(curl);
         return 2;
     }
 
-    if (onas_sendln(curl, "zVERSION", 9, timeout)) {
+    if (onas_sendln(curl, zVERSION, sizeof(zVERSION), timeout)) {
         curl_easy_cleanup(curl);
         return 2;
     }
 
     while ((len = onas_recvln(&rcv, &buff, NULL, timeout))) {
         if (len == -1) {
-            logg("*ClamClient: clamd did not respond with version information\n");
+            logg(LOGG_DEBUG, "ClamClient: clamd did not respond with version information\n");
             break;
         }
         printf("%s\n", buff);
@@ -560,7 +551,7 @@ int onas_client_scan(const char *tcpaddr, int64_t portnum, int32_t scantype, uin
 
     curlcode = onas_curl_init(&curl, tcpaddr, portnum, timeout);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not init curl for scanning, %s\n", curl_easy_strerror(curlcode));
+        logg(LOGG_ERROR, "ClamClient: could not init curl for scanning, %s\n", curl_easy_strerror(curlcode));
         /* curl cleanup done in onas_curl_init on error */
         return CL_ECREAT;
     }
@@ -568,20 +559,21 @@ int onas_client_scan(const char *tcpaddr, int64_t portnum, int32_t scantype, uin
     curlcode = curl_easy_perform(curl);
     if (CURLE_OK != curlcode) {
         if (!disconnected) {
-            logg("!ClamClient: Connection to clamd failed, %s.\n", curl_easy_strerror(curlcode));
+            logg(LOGG_ERROR, "ClamClient: Connection to clamd failed, %s.\n", curl_easy_strerror(curlcode));
             disconnected = true;
         }
+        curl_easy_cleanup(curl);
         return CL_ECREAT;
     }
     if (disconnected) {
-        logg("~ClamClient: Connection to clamd re-established.\n");
+        logg(LOGG_INFO, "ClamClient: Connection to clamd re-established.\n");
         disconnected = false;
     }
 
     if ((ret = onas_dsresult(curl, scantype, maxstream, fname, fd, timeout, &ret, err, ret_code)) >= 0) {
         *infected = ret;
     } else {
-        logg("*ClamClient: connection could not be established ... return code %d\n", *ret_code);
+        logg(LOGG_DEBUG, "ClamClient: connection could not be established ... return code %d\n", *ret_code);
         errors = 1;
     }
 
