@@ -41,7 +41,7 @@
 #define CHECK_ARCH(a) \
     if (!strcmp(TARGET_ARCH_TYPE, #a)) env->arch = arch_##a
 
-extern int have_clamjit;
+extern bool have_clamjit(void);
 
 static void cli_print_environment(struct cli_environment *env)
 {
@@ -193,6 +193,7 @@ void cli_detect_environment(struct cli_environment *env)
     CHECK_ARCH(i386);
     else CHECK_ARCH(x86_64);
     else if (!strcmp(TARGET_ARCH_TYPE, "amd64")) env->arch = arch_x86_64;
+    else if (!strcmp(TARGET_ARCH_TYPE, "AMD64")) env->arch = arch_x86_64;
     else if (!strcmp(TARGET_ARCH_TYPE, "ppc")) env->arch   = arch_ppc32; /* llvm will fix ppc64 */
     else CHECK_ARCH(arm);
     else CHECK_ARCH(sparc);
@@ -274,7 +275,7 @@ void cli_detect_environment(struct cli_environment *env)
 #endif
     env->cpp_version = 0;
 
-    env->has_jit_compiled = have_clamjit;
+    env->has_jit_compiled = have_clamjit();
 
     /* engine */
     env->functionality_level = cl_retflevel();
