@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -132,6 +132,9 @@ static struct dconf_module modules[] = {
     {"OTHER", "PDFNAMEOBJ", OTHER_CONF_PDFNAMEOBJ, 1},
     {"OTHER", "PRTNINTXN", OTHER_CONF_PRTNINTXN, 1},
     {"OTHER", "LZW", OTHER_CONF_LZW, 1},
+    {"OTHER", "GIF", OTHER_CONF_GIF, 1},
+    {"OTHER", "PNG", OTHER_CONF_PNG, 1},
+    {"OTHER", "TIFF", OTHER_CONF_TIFF, 1},
 
     {"PHISHING", "ENGINE", PHISHING_CONF_ENGINE, 1},
     {"PHISHING", "ENTCONV", PHISHING_CONF_ENTCONV, 1},
@@ -167,27 +170,21 @@ struct cli_dconf *cli_dconf_init(void)
         if (!strcmp(modules[i].mname, "PE")) {
             if (modules[i].state)
                 dconf->pe |= modules[i].bflag;
-
         } else if (!strcmp(modules[i].mname, "ELF")) {
             if (modules[i].state)
                 dconf->elf |= modules[i].bflag;
-
         } else if (!strcmp(modules[i].mname, "MACHO")) {
             if (modules[i].state)
                 dconf->macho |= modules[i].bflag;
-
         } else if (!strcmp(modules[i].mname, "ARCHIVE")) {
             if (modules[i].state)
                 dconf->archive |= modules[i].bflag;
-
         } else if (!strcmp(modules[i].mname, "DOCUMENT")) {
             if (modules[i].state)
                 dconf->doc |= modules[i].bflag;
-
         } else if (!strcmp(modules[i].mname, "MAIL")) {
             if (modules[i].state)
                 dconf->mail |= modules[i].bflag;
-
         } else if (!strcmp(modules[i].mname, "OTHER")) {
             if (modules[i].state)
                 dconf->other |= modules[i].bflag;
