@@ -20,7 +20,9 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *  MA 02110-1301, USA.
  */
+#if HAVE_CONFIG_H
 #include "clamav-config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -83,7 +85,7 @@ static inline unsigned char* u16_normalize(uint16_t u16, unsigned char* out, con
             return NULL;
         }
         /* inline version of
-		 * out += snprintf(out, max_num_length, "&#x%x;", u16) */
+         * out += snprintf(out, max_num_length, "&#x%x;", u16) */
         out[0] = '&';
         out[1] = '#';
         out[2] = 'x';
@@ -112,7 +114,7 @@ const char* entity_norm(struct entity_conv* conv, const unsigned char* entity)
 {
     struct cli_element* e = cli_hashtab_find(&entities_htable, (const char*)entity, strlen((const char*)entity));
     if (e && e->key) {
-        unsigned char* out = u16_normalize(e->data, conv->entity_buff, sizeof(conv->entity_buff) - 1);
+        unsigned char* out = u16_normalize((uint16_t)e->data, conv->entity_buff, sizeof(conv->entity_buff) - 1);
         if (out) {
             *out++ = '\0';
             return (const char*)conv->entity_buff;
@@ -127,7 +129,7 @@ static size_t encoding_bytes(const char* fromcode, enum encodings* encoding)
     /* special case for these unusual byteorders */
     struct cli_element* e = cli_hashtab_find(&aliases_htable, fromcode, strlen(fromcode));
     if (e && e->key) {
-        *encoding = e->data;
+        *encoding = (enum encodings)e->data;
     } else {
         *encoding = E_OTHER;
     }
@@ -301,8 +303,8 @@ static int iconv(iconv_t iconv_struct, char** inbuf, size_t* inbytesleft,
                         output[j++] = 0xff;
 
                         /*out[j++] = ((input[i] & 0x07) << 2) | ((input[i+1] >> 4) & 0x3);
-							out[j++] = (input[i+1] << 4) | ((input[i+2] >> 2) & 0x0F);
-							out[j++] = (input[i+2] << 6) | (input[i+2] & 0x3F);*/
+                                                        out[j++] = (input[i+1] << 4) | ((input[i+2] >> 2) & 0x0F);
+                                                        out[j++] = (input[i+2] << 6) | (input[i+2] & 0x3F);*/
                     } else {
                         cli_dbgmsg(MODULE_NAME "invalid UTF8 character encountered\n");
                         break;
@@ -350,7 +352,7 @@ static inline const char* detect_encoding(const unsigned char* bom, uint8_t* bom
     int has_bom          = 0;
     uint8_t enc_bytes    = 1; /* default is UTF8, which has a minimum of 1 bytes */
     /* undecided 32-bit encodings are treated as ucs4, and
-	 * 16 bit as utf16*/
+     * 16 bit as utf16*/
     switch (bom[0]) {
         case 0x00:
             if (bom[1] == 0x00) {
@@ -515,8 +517,8 @@ struct iconv_cache {
 static void iconv_cache_init(struct iconv_cache* cache)
 {
     /*	cache->tab = NULL;
-	cache->len = 0;
-	cache->used = 0; - already done by memset*/
+        cache->len = 0;
+        cache->used = 0; - already done by memset*/
     cli_dbgmsg(MODULE_NAME "Initializing iconv pool:%p\n", (void*)cache);
     cli_hashtab_init(&cache->hashtab, 32);
 }
@@ -639,14 +641,14 @@ static iconv_t iconv_open_cached(const char* fromcode)
     }
 
     e = cli_hashtab_find(&cache->hashtab, fromcode, fromcode_len);
-    if (e && (e->data < 0 || (size_t)e->data > cache->len)) {
+    if (e && ((size_t)e->data == (size_t)-1 || (size_t)e->data > cache->len)) {
         e = NULL;
     }
     if (e) {
         size_t dummy_in, dummy_out;
         /* reset state */
-        iconv(cache->tab[e->data], NULL, &dummy_in, NULL, &dummy_out);
-        return cache->tab[e->data];
+        iconv(cache->tab[(size_t)e->data], NULL, &dummy_in, NULL, &dummy_out);
+        return cache->tab[(size_t)e->data];
     }
     cli_dbgmsg(MODULE_NAME "iconv not found in cache, for encoding:%s\n", fromcode);
     iconv_struct = iconv_open("UTF-16BE", (const char*)fromcode);
@@ -664,7 +666,7 @@ static iconv_t iconv_open_cached(const char* fromcode)
             }
         }
 
-        cli_hashtab_insert(&cache->hashtab, fromcode, fromcode_len, idx);
+        cli_hashtab_insert(&cache->hashtab, fromcode, fromcode_len, (const cli_element_data)idx);
         cache->tab[idx] = iconv_struct;
         cli_dbgmsg(MODULE_NAME "iconv_open(),for:%s -> %p\n", fromcode, (void*)cache->tab[idx]);
         return cache->tab[idx];
@@ -687,7 +689,7 @@ static int in_iconv_u16(const m_area_t* in_m_area, iconv_t* iconv_struct, m_area
     }
     /* convert encoding conv->tmp_area. conv->out_area */
     alignfix = inleft % 4; /* iconv gives an error if we give him 3 bytes to convert,
-			       and we are using ucs4, ditto for utf16, and 1 byte*/
+                               and we are using ucs4, ditto for utf16, and 1 byte*/
     inleft -= alignfix;
 
     if (!inleft && alignfix) {
@@ -716,8 +718,8 @@ static int in_iconv_u16(const m_area_t* in_m_area, iconv_t* iconv_struct, m_area
             continue;
         }
         /*cli_dbgmsg(MODULE_NAME "resuming (inleft:%lu, outleft:%lu, inpos:%ld, %ld)\n",
-					inleft, outleft, input - (char*)in_m_area->buffer,
-					out - (char*)out_m_area->buffer);*/
+                                        inleft, outleft, input - (char*)in_m_area->buffer,
+                                        out - (char*)out_m_area->buffer);*/
         /* output raw byte, and resume at next byte */
         if (outleft < 2) break;
         outleft -= 2;
@@ -850,41 +852,69 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                 /*
                  * First, Convert from codepage -> UCS-2 LE with MultiByteToWideChar(codepage)
                  */
-                cchWideChar = MultiByteToWideChar(
-                    codepage,
-                    0,
-                    in,
-                    in_size,
-                    NULL,
-                    0);
-                if (0 == cchWideChar) {
-                    cli_dbgmsg("cli_codepage_to_utf8: failed to determine string size needed for ansi to widechar conversion.\n");
-                    status = CL_EPARSE;
-                    goto done;
-                }
+                if (CODEPAGE_UTF16_BE == codepage) {
+                    /*
+                     * MultiByteToWideChar() does not support conversions from UTF-16BE to UTF-16LE.
+                     * However, conversion is simply a matter of swapping the bytes for each WCHAR_T.
+                     * See: https://stackoverflow.com/questions/29054217/multibytetowidechar-for-unicode-code-pages-1200-1201-12000-12001
+                     */
+                    int i = 0;
 
-                lpWideCharStr = cli_malloc((cchWideChar + 1) * sizeof(WCHAR));
-                if (NULL == lpWideCharStr) {
-                    cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char string.\n");
-                    status = CL_EMEM;
-                    goto done;
-                }
+                    uint16_t* pCodeUnits = (uint16_t*)in;
+                    cchWideChar          = (int)in_size / 2;
 
-                cchWideChar = MultiByteToWideChar(
-                    codepage,
-                    0,
-                    in,
-                    in_size,
-                    lpWideCharStr,
-                    cchWideChar + 1);
-                if (0 == cchWideChar) {
-                    cli_dbgmsg("cli_codepage_to_utf8: failed to convert multibyte string to widechars.\n");
-                    status = CL_EPARSE;
-                    goto done;
-                }
+                    lpWideCharStr = cli_malloc((cchWideChar) * sizeof(WCHAR)); /* No need for a null terminator here, we'll deal with the exact size */
+                    if (NULL == lpWideCharStr) {
+                        cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char string.\n");
+                        status = CL_EMEM;
+                        goto done;
+                    }
 
-                in      = (char*)lpWideCharStr;
-                in_size = cchWideChar;
+                    for (i = 0; i < cchWideChar; i++) {
+                        lpWideCharStr[i] = (WCHAR)(((pCodeUnits[i] << 8) & 0xFF00) |
+                                                   ((pCodeUnits[i] >> 8) & 0x00FF));
+                    }
+                    in = (char*)lpWideCharStr;
+                    /* in_size didn't change */
+
+                } else {
+                    cchWideChar = MultiByteToWideChar(
+                        codepage,
+                        0,
+                        in,
+                        (int)in_size,
+                        NULL,
+                        0);
+                    if (0 == cchWideChar) {
+                        DWORD error = GetLastError();
+                        cli_dbgmsg("cli_codepage_to_utf8: failed to determine string size needed for ansi to widechar conversion: %d.\n", error);
+                        status = CL_EPARSE;
+                        goto done;
+                    }
+
+                    lpWideCharStr = cli_malloc((cchWideChar) * sizeof(WCHAR)); /* No need for a null terminator here, we'll deal with the exact size */
+                    if (NULL == lpWideCharStr) {
+                        cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char string.\n");
+                        status = CL_EMEM;
+                        goto done;
+                    }
+
+                    cchWideChar = MultiByteToWideChar(
+                        codepage,
+                        0,
+                        in,
+                        (int)in_size,
+                        lpWideCharStr,
+                        cchWideChar);
+                    if (0 == cchWideChar) {
+                        cli_dbgmsg("cli_codepage_to_utf8: failed to convert multibyte string to widechars.\n");
+                        status = CL_EPARSE;
+                        goto done;
+                    }
+
+                    in      = (char*)lpWideCharStr;
+                    in_size = cchWideChar * sizeof(WCHAR);
+                }
             }
 
             /*
@@ -894,7 +924,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                 CP_UTF8,
                 0,
                 (LPCWCH)in,
-                in_size / sizeof(WCHAR),
+                (int)in_size / sizeof(WCHAR),
                 NULL,
                 0,
                 NULL,
@@ -905,7 +935,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                 goto done;
             }
 
-            out_utf8 = cli_malloc(out_utf8_size + 1);
+            out_utf8 = cli_malloc(out_utf8_size + 1); /* Add a null terminator to this string */
             if (NULL == out_utf8) {
                 cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char to utf-8 string.\n");
                 status = CL_EMEM;
@@ -916,9 +946,9 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                 CP_UTF8,
                 0,
                 (LPCWCH)in,
-                in_size / sizeof(WCHAR),
+                (int)in_size / sizeof(WCHAR),
                 out_utf8,
-                out_utf8_size,
+                (int)out_utf8_size,
                 NULL,
                 NULL);
             if (0 == out_utf8_size) {
@@ -974,8 +1004,13 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
 
                 conv = iconv_open("UTF-8//TRANSLIT", encoding);
                 if (conv == (iconv_t)-1) {
-                    cli_warnmsg("cli_codepage_to_utf8: Failed to open iconv.\n");
-                    goto done;
+                    // Try again w/out the //TRANSLIT, required because musl doesn't supprot it.
+                    // See: https://github.com/akrennmair/newsbeuter/issues/364#issuecomment-250208235
+                    conv = iconv_open("UTF-8", encoding);
+                    if (conv == (iconv_t)-1) {
+                        cli_warnmsg("cli_codepage_to_utf8: Failed to open iconv.\n");
+                        goto done;
+                    }
                 }
 
                 iconvRet = iconv(conv, &inbuf, &inbufsize, &out_utf8_index, &outbytesleft);
@@ -992,7 +1027,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                             cli_warnmsg("cli_codepage_to_utf8: iconv error: An invalid multibyte sequence has been encountered in the input.\n");
                             break;
                         case EINVAL:
-                            cli_warnmsg("cli_codepage_to_utf8: iconv error: An incomplete multibyte sequence has been encountered in the input.\n");
+                            cli_dbgmsg("cli_codepage_to_utf8: iconv error: An incomplete multibyte sequence has been encountered in the input.\n");
                             break;
                         default:
                             cli_warnmsg("cli_codepage_to_utf8: iconv error: Unexpected error code %d.\n", errno);
