@@ -46,6 +46,7 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
     uint16_t i, num_entries;
     struct tiff_ifd entry;
     size_t value_size;
+    uint32_t last_offset = 0;
 
     cli_dbgmsg("in cli_parsetiff()\n");
 
@@ -77,18 +78,17 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
     /* acquire offset of first IFD */
     if (fmap_readn(map, &offset, offset, 4) != 4) {
         cli_dbgmsg("cli_parsetiff: Failed to acquire offset of first IFD, file appears to be truncated.\n");
-        cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingFirstIFDOffset");
-        status = CL_EPARSE;
+        status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingFirstIFDOffset");
         goto done;
     }
+    /* offset of the first IFD */
     offset = tiff32_to_host(big_endian, offset);
 
     cli_dbgmsg("cli_parsetiff: first IFD located @ offset %u\n", offset);
 
     if (!offset) {
         cli_errmsg("cli_parsetiff: Invalid offset for first IFD\n");
-        cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.InvalidIFDOffset");
-        status = CL_EPARSE;
+        status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.InvalidIFDOffset");
         goto done;
     }
 
@@ -97,8 +97,7 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
         /* acquire number of directory entries in current IFD */
         if (fmap_readn(map, &num_entries, offset, 2) != 2) {
             cli_dbgmsg("cli_parsetiff: Failed to acquire number of directory entries in current IFD, file appears to be truncated.\n");
-            cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingNumIFDDirectoryEntries");
-            status = CL_EPARSE;
+            status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingNumIFDDirectoryEntries");
             goto done;
         }
         offset += 2;
@@ -110,8 +109,7 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
         for (i = 0; i < num_entries; i++) {
             if (fmap_readn(map, &entry, offset, sizeof(entry)) != sizeof(entry)) {
                 cli_dbgmsg("cli_parsetiff: Failed to read next IFD entry, file appears to be truncated.\n");
-                cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingIFDEntry");
-                status = CL_EPARSE;
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingIFDEntry");
                 goto done;
             }
             offset += sizeof(entry);
@@ -121,7 +119,7 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
             entry.numval = tiff32_to_host(big_endian, entry.numval);
             entry.value  = tiff32_to_host(big_endian, entry.value);
 
-            //cli_dbgmsg("%02u: %u %u %u %u\n", i, entry.tag, entry.type, entry.numval, entry.value);
+            // cli_dbgmsg("%02u: %u %u %u %u\n", i, entry.tag, entry.type, entry.numval, entry.value);
 
             value_size = entry.numval;
             switch (entry.type) {
@@ -173,7 +171,7 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
                 if (entry.value + value_size > map->len) {
                     cli_warnmsg("cli_parsetiff: TFD entry field %u exceeds bounds of TIFF file [%llu > %llu]\n",
                                 i, (long long unsigned)(entry.value + value_size), (long long unsigned)map->len);
-                    status = cli_append_virus(ctx, "Heuristics.Broken.Media.TIFF.OutOfBoundsAccess");
+                    status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.OutOfBoundsAccess");
                     goto done;
                 }
             }
@@ -181,14 +179,24 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
 
         ifd_count++;
 
+        last_offset = offset;
+
         /* acquire next IFD location, gets 0 if last IFD */
         if (fmap_readn(map, &offset, offset, sizeof(offset)) != sizeof(offset)) {
             cli_dbgmsg("cli_parsetiff: Failed to aquire next IFD location, file appears to be truncated.\n");
-            cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingChunkCRC");
-            status = CL_EPARSE;
+            status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingChunkCRC");
             goto done;
         }
         offset = tiff32_to_host(big_endian, offset);
+
+        if (offset) {
+            /*If the offsets are not in order, that is suspicious.*/
+            if (last_offset >= offset) {
+                cli_dbgmsg("cli_parsetiff: Next offset is before current offset, file appears to be malformed.\n");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Media.TIFF.OutOfOrderIFDOffset");
+                goto done;
+            }
+        }
     } while (offset);
 
     cli_dbgmsg("cli_parsetiff: examined %u IFD(s)\n", ifd_count);
@@ -196,10 +204,6 @@ cl_error_t cli_parsetiff(cli_ctx *ctx)
     status = CL_CLEAN;
 
 done:
-    if (status == CL_EPARSE) {
-        /* We added with cli_append_possibly_unwanted so it will alert at the end if nothing else matches. */
-        status = CL_CLEAN;
-    }
 
     return status;
 }
