@@ -115,10 +115,11 @@ static cl_error_t unz(
     cli_ctx *ctx,
     char *tmpd,
     zip_cb zcb,
-    const char *original_filename)
+    const char *original_filename,
+    bool decrypted)
 {
-    char obuf[BUFSIZ];
-    char *tempfile = NULL;
+    char obuf[BUFSIZ] = {0};
+    char *tempfile    = NULL;
     int out_file, ret = CL_CLEAN;
     int res        = 1;
     size_t written = 0;
@@ -146,7 +147,8 @@ static cl_error_t unz(
             if (csize < usize) {
                 unsigned int fake = *num_files_unzipped + 1;
                 cli_dbgmsg("cli_unzip: attempting to inflate stored file with inconsistent size\n");
-                if ((ret = unz(src, csize, usize, ALG_DEFLATE, 0, &fake, ctx, tmpd, zcb, original_filename)) == CL_CLEAN) {
+                if (CL_CLEAN == (ret = unz(src, csize, usize, ALG_DEFLATE, 0, &fake, ctx,
+                                           tmpd, zcb, original_filename, decrypted))) {
                     (*num_files_unzipped)++;
                     res = fake - (*num_files_unzipped);
                 } else
@@ -154,7 +156,8 @@ static cl_error_t unz(
             }
             if (res == 1) {
                 if (ctx->engine->maxfilesize && csize > ctx->engine->maxfilesize) {
-                    cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", (long unsigned int)ctx->engine->maxfilesize);
+                    cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n",
+                               (long unsigned int)ctx->engine->maxfilesize);
                     csize = ctx->engine->maxfilesize;
                 }
                 if (cli_writen(out_file, src, csize) != csize)
@@ -354,7 +357,7 @@ static cl_error_t unz(
             close(out_file);
             return CL_ESEEK;
         }
-        ret = zcb(out_file, tempfile, ctx, original_filename);
+        ret = zcb(out_file, tempfile, ctx, original_filename, decrypted);
         close(out_file);
         if (!ctx->engine->keeptmp)
             if (cli_unlink(tempfile)) ret = CL_EUNLINK;
@@ -556,7 +559,8 @@ static inline cl_error_t zdecrypt(
             }
 
             /* call unz on decrypted output */
-            ret = unz(dcypt_zip, csize - SIZEOF_ENCRYPTION_HEADER, usize, LOCAL_HEADER_method, LOCAL_HEADER_flags, num_files_unzipped, ctx, tmpd, zcb, original_filename);
+            ret = unz(dcypt_zip, csize - SIZEOF_ENCRYPTION_HEADER, usize, LOCAL_HEADER_method, LOCAL_HEADER_flags,
+                      num_files_unzipped, ctx, tmpd, zcb, original_filename, true);
 
             /* clean-up and return */
             funmap(dcypt_map);
@@ -618,7 +622,6 @@ static unsigned int parse_local_file_header(
     char name[256];
     char *original_filename = NULL;
     uint32_t csize, usize;
-    int virus_found                          = 0;
     unsigned int size_of_fileheader_and_data = 0;
 
     if (!(local_header = fmap_need_off(map, loff, SIZEOF_LOCAL_HEADER))) {
@@ -637,6 +640,8 @@ static unsigned int parse_local_file_header(
     zip = local_header + SIZEOF_LOCAL_HEADER;
     zsize -= SIZEOF_LOCAL_HEADER;
 
+    memset(name, '\0', 256);
+
     if (zsize <= LOCAL_HEADER_flen) {
         cli_dbgmsg("cli_unzip: local header - fname out of file\n");
         fmap_unneed_off(map, loff, SIZEOF_LOCAL_HEADER);
@@ -664,9 +669,7 @@ static unsigned int parse_local_file_header(
     /* Scan file header metadata. */
     if (cli_matchmeta(ctx, name, LOCAL_HEADER_csize, LOCAL_HEADER_usize, (LOCAL_HEADER_flags & F_ENCR) != 0, file_count, LOCAL_HEADER_crc32, NULL) == CL_VIRUS) {
         *ret = CL_VIRUS;
-        if (!SCAN_ALLMATCHES)
-            goto done;
-        virus_found = 1;
+        goto done;
     }
 
     if (LOCAL_HEADER_flags & F_MSKED) {
@@ -679,13 +682,12 @@ static unsigned int parse_local_file_header(
     if (detect_encrypted && (LOCAL_HEADER_flags & F_ENCR) && SCAN_HEURISTIC_ENCRYPTED_ARCHIVE) {
         cl_error_t fp_check;
         cli_dbgmsg("cli_unzip: Encrypted files found in archive.\n");
-        fp_check = cli_append_virus(ctx, "Heuristics.Encrypted.Zip");
-        if ((fp_check == CL_VIRUS && !SCAN_ALLMATCHES) || fp_check != CL_CLEAN) {
+        fp_check = cli_append_potentially_unwanted(ctx, "Heuristics.Encrypted.Zip");
+        if (fp_check != CL_SUCCESS) {
             *ret = fp_check;
             fmap_unneed_off(map, loff, SIZEOF_LOCAL_HEADER);
             goto done;
         }
-        virus_found = 1;
     }
 
     if (LOCAL_HEADER_flags & F_USEDD) {
@@ -726,7 +728,8 @@ static unsigned int parse_local_file_header(
                     *ret = zdecrypt(zip, csize, usize, local_header, num_files_unzipped, ctx, tmpd, zcb, original_filename);
             } else {
                 if (fmap_need_ptr_once(map, zip, csize))
-                    *ret = unz(zip, csize, usize, LOCAL_HEADER_method, LOCAL_HEADER_flags, num_files_unzipped, ctx, tmpd, zcb, original_filename);
+                    *ret = unz(zip, csize, usize, LOCAL_HEADER_method, LOCAL_HEADER_flags, num_files_unzipped,
+                               ctx, tmpd, zcb, original_filename, false);
             }
         } else {
             if ((NULL == original_filename) ||
@@ -775,9 +778,6 @@ done:
         free(original_filename);
     }
 
-    if ((NULL != ret) && (0 != virus_found))
-        *ret = CL_VIRUS;
-
     return size_of_fileheader_and_data;
 }
 
@@ -814,10 +814,16 @@ parse_central_directory_file_header(
     char name[256];
     int last                      = 0;
     const uint8_t *central_header = NULL;
-    int virus_found               = 0;
 
     *ret = CL_EPARSE;
 
+    if (cli_checktimelimit(ctx) != CL_SUCCESS) {
+        cli_dbgmsg("cli_unzip: central header - Time limit reached (max: %u)\n", ctx->engine->maxscantime);
+        last = 1;
+        *ret = CL_ETIMEOUT;
+        goto done;
+    }
+
     if (!(central_header = fmap_need_off(map, coff, SIZEOF_CENTRAL_HEADER)) || CENTRAL_HEADER_magic != ZIP_MAGIC_CENTRAL_DIRECTORY_RECORD_BEGIN) {
         if (central_header) {
             fmap_unneed_ptr(map, central_header, SIZEOF_CENTRAL_HEADER);
@@ -852,12 +858,9 @@ parse_central_directory_file_header(
 
     /* requests do not supply a ctx; also prevent multiple scans */
     if (ctx && (CL_VIRUS == cli_matchmeta(ctx, name, CENTRAL_HEADER_csize, CENTRAL_HEADER_usize, (CENTRAL_HEADER_flags & F_ENCR) != 0, file_count, CENTRAL_HEADER_crc32, NULL))) {
-        virus_found = 1;
-
-        if (!SCAN_ALLMATCHES) {
-            last = 1;
-            goto done;
-        }
+        last = 1;
+        *ret = CL_VIRUS;
+        goto done;
     }
 
     if (zsize - coff <= CENTRAL_HEADER_extra_len && !last) {
@@ -909,9 +912,6 @@ parse_central_directory_file_header(
     }
 
 done:
-    if (virus_found == 1)
-        *ret = CL_VIRUS;
-
     if (NULL != central_header) {
         fmap_unneed_ptr(map, central_header, SIZEOF_CENTRAL_HEADER);
     }
@@ -925,7 +925,7 @@ done:
  * @param first
  * @param second
  * @return int 1 if first record's offset is higher than second's.
- * @return int 0 if first and second reocrd offsets are equal.
+ * @return int 0 if first and second record offsets are equal.
  * @return int -1 if first record's offset is less than second's.
  */
 static int sort_by_file_offset(const void *first, const void *second)
@@ -984,7 +984,6 @@ cl_error_t index_the_central_directory(
     struct zip_record *curr_record   = NULL;
     struct zip_record *prev_record   = NULL;
     uint32_t num_overlapping_files   = 0;
-    int virus_found                  = 0;
     bool exceeded_max_files          = false;
 
     if (NULL == catalogue || NULL == num_records) {
@@ -1028,12 +1027,8 @@ cl_error_t index_the_central_directory(
         }
 
         if (ret == CL_VIRUS) {
-            if (SCAN_ALLMATCHES)
-                virus_found = 1;
-            else {
-                status = CL_VIRUS;
-                goto done;
-            }
+            status = CL_VIRUS;
+            goto done;
         }
 
         index++;
@@ -1047,7 +1042,7 @@ cl_error_t index_the_central_directory(
         /* stop checking file entries if we'll exceed maxfiles */
         if (ctx->engine->maxfiles && records_count >= ctx->engine->maxfiles) {
             cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
-            cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
+            cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
             exceeded_max_files = true; // Set a bool so we can return the correct status code later.
                                        // We still need to scan the files we found while reviewing the file records up to this limit.
             break;
@@ -1081,12 +1076,8 @@ cl_error_t index_the_central_directory(
     } while (1);
 
     if (ret == CL_VIRUS) {
-        if (SCAN_ALLMATCHES)
-            virus_found = 1;
-        else {
-            status = CL_VIRUS;
-            goto done;
-        }
+        status = CL_VIRUS;
+        goto done;
     }
 
     if (records_count > 1) {
@@ -1134,7 +1125,7 @@ cl_error_t index_the_central_directory(
 
                     if (ZIP_MAX_NUM_OVERLAPPING_FILES < num_overlapping_files) {
                         if (SCAN_HEURISTICS) {
-                            status = cli_append_virus(ctx, "Heuristics.Zip.OverlappingFiles");
+                            status = cli_append_potentially_unwanted(ctx, "Heuristics.Zip.OverlappingFiles");
                         } else {
                             status = CL_EFORMAT;
                         }
@@ -1169,12 +1160,11 @@ done:
             free(zip_catalogue);
             zip_catalogue = NULL;
         }
-    }
 
-    if (virus_found)
-        status = CL_VIRUS;
-    else if (exceeded_max_files)
-        status = CL_EMAXFILES;
+        if (exceeded_max_files) {
+            status = CL_EMAXFILES;
+        }
+    }
 
     return status;
 }
@@ -1187,7 +1177,6 @@ cl_error_t cli_unzip(cli_ctx *ctx)
     fmap_t *map = ctx->fmap;
     char *tmpd  = NULL;
     const char *ptr;
-    int virus_found = 0;
 #if HAVE_JSON
     int toval = 0;
 #endif
@@ -1233,11 +1222,7 @@ cl_error_t cli_unzip(cli_ctx *ctx)
             &zip_catalogue,
             &records_count);
         if (CL_SUCCESS != ret) {
-            if (CL_VIRUS == ret && SCAN_ALLMATCHES)
-                virus_found = 1;
-            else {
-                goto done;
-            }
+            goto done;
         }
 
         /*
@@ -1282,7 +1267,8 @@ cl_error_t cli_unzip(cli_ctx *ctx)
                         ctx,
                         tmpd,
                         zip_scan_cb,
-                        zip_catalogue[i].original_filename);
+                        zip_catalogue[i].original_filename,
+                        false);
             }
 
             file_count++;
@@ -1294,13 +1280,14 @@ cl_error_t cli_unzip(cli_ctx *ctx)
                 //   so we will also check and update the limits for the actual number of scanned
                 //   files inside cli_magic_scan()
                 cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
-                cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
+                cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
                 ret = CL_EMAXFILES;
             }
 
             if (cli_checktimelimit(ctx) != CL_SUCCESS) {
                 cli_dbgmsg("cli_unzip: Time limit reached (max: %u)\n", ctx->engine->maxscantime);
                 ret = CL_ETIMEOUT;
+                goto done;
             }
 
 #if HAVE_JSON
@@ -1308,22 +1295,21 @@ cl_error_t cli_unzip(cli_ctx *ctx)
                 ret = CL_ETIMEOUT;
             }
 #endif
-            if (ret != CL_CLEAN) {
-                if (ret == CL_VIRUS && SCAN_ALLMATCHES) {
-                    ret         = CL_CLEAN;
-                    virus_found = 1;
-                } else {
-                    break;
-                }
+            if (ret != CL_SUCCESS) {
+                break;
             }
         }
     } else {
         cli_dbgmsg("cli_unzip: central not found, using localhdrs\n");
     }
 
-    if (virus_found == 1) {
-        ret = CL_VIRUS;
+    if (CL_SUCCESS != ret) {
+        // goto done right away if there was a timeout, an alert, etc.
+        // This is slightly redundant since the while loop will only happen
+        // if ret == CL_SUCCESS but it's more explicit.
+        goto done;
     }
+
     if (0 < num_files_unzipped && num_files_unzipped <= (file_count / 4)) { /* FIXME: make up a sane ratio or remove the whole logic */
         file_count = 0;
         while ((ret == CL_CLEAN) &&
@@ -1342,10 +1328,7 @@ cl_error_t cli_unzip(cli_ctx *ctx)
                                                      NULL)))) {
             file_count++;
             lhoff += coff;
-            if (SCAN_ALLMATCHES && ret == CL_VIRUS) {
-                ret         = CL_CLEAN;
-                virus_found = 1;
-            }
+
             if (ctx->engine->maxfiles && num_files_unzipped >= ctx->engine->maxfiles) {
                 // Note: this check piggybacks on the MaxFiles setting, but is not actually
                 //   scanning these files or incrementing the ctx->scannedfiles count
@@ -1353,7 +1336,7 @@ cl_error_t cli_unzip(cli_ctx *ctx)
                 //   so we will also check and update the limits for the actual number of scanned
                 //   files inside cli_magic_scan()
                 cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
-                cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
+                cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
                 ret = CL_EMAXFILES;
             }
 #if HAVE_JSON
@@ -1385,9 +1368,6 @@ done:
         free(tmpd);
     }
 
-    if (ret == CL_CLEAN && virus_found)
-        ret = CL_VIRUS;
-
     return ret;
 }
 
@@ -1514,7 +1494,7 @@ cl_error_t unzip_search(cli_ctx *ctx, fmap_t *map, struct zip_requests *requests
                 // Note: this check piggybacks on the MaxFiles setting, but is not actually
                 //   scanning these files or incrementing the ctx->scannedfiles count
                 cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
-                cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
+                cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
                 ret = CL_EMAXFILES;
             }
 #if HAVE_JSON
